{ Program bada przynaleno punktu do odcinka }
program punkt_do_odcinka;
uses
  Crt, Math;

type
  TPunkt=Record    { rekord przechowujcy dane o punkcie }
    x : Real;
    y : Real;
  end;

  { nachylenie to stosunek wzniesienia do przesunicia: dy/dx }
  TNachyl = Record
    dy : Real; { rnica w pionie - wzniesienie }
    dx : Real; { rnica w poziomie - przesuniecie }
  end;

  TOdcinek=Record
    p : TPunkt;    { pocztek odcinka }
    q : TPunkt;    { koniec   odcinka }
    n : TNachyl;   { nachylenie n.dy=p.y-q.y, n.dx=p.x-q.x }
  end;

{ Procedura pomocnicza, ktra zbiera info o punkcie w jednym rekordzie }
procedure punkt(x,y : Real; var punkt : TPunkt);
begin
  punkt.x:= x;
  punkt.y:= y;
end;

{ Procedura pomocnicza, ktra zbiera info o odcinku w jednym rekordzie }
procedure odcinek(p,q : TPunkt; n : TNachyl; var odcinek : TOdcinek);
begin
  odcinek.p.x:=p.x;
  odcinek.p.y:=p.y;
  odcinek.q.x:=q.x;
  odcinek.q.y:=q.y;
  odcinek.n.dy:=n.dy;
  odcinek.n.dx:=n.dx;
end;

{ Funkcja oblicza odlego miedzy dwoma punktami - dugo odcinka }
function odl_pkt(p1,p2 : TPunkt) : Real;
var
  odlX,         { przesunicie }
  odlY : Real;  { wzniesienie }

begin
  odlX:=p1.x-p2.x;
  odlY:=p1.y-p2.y;
  odl_pkt:=sqrt(odlY*odlY+odlX*odlX);
end;

{ Funkcja sprawdza czy poprawne jest nachylenie }
{ Parametry wejciowe: dy - rnica w pionie, dx - rnica w poziomie }
{ Wynik:
{ 0 - nachylenie o poprawnych wartociach, rnych od zera }
{ 255 - para nie reprezentuje nachylenia - brak przesuni }
{ 1 - brak wzniesienia - nachylenie zerowe - odcinek poziomy }
{ 2 - brak przesunicia - nachylenie nie istnieje - odcinek pionowy }
function sprawdz_nachyl (dy, dx : Real) : Byte;
var
  wynik : Byte;
begin
  wynik:=0;
  if (dy=0) and (dx=0) then wynik:=255 { brak przesuni dy i dx }
  else
    if (dy=0) then wynik:=1 { pozioma - brak wzniesienia - nachylenie zerowe }
    else
      if(dx=0) then wynik:=2; { pionowa - brak przesunicia - nachylenie nie istnieje }

  sprawdz_nachyl:=wynik;
end;

{ Funkcja na podstawie wartoci rnicy pionowej i rnicy poziomej, ktre }
{ s zapisane w rekordzie nachyl oblicza wspczynnik kierunkowy prostej }
function dajKier ( nachyl : TNachyl ) : Real;
begin
  dajKier:=nachyl.dy/nachyl.dx;
end;

{ Funkcja oblicza miejsce przecicia prostej zawierajcej odcinek z osia y }
{ czyli warto b z rwnania  y=m*x+b,=> b=y-m*x }
function  dajB (x, y, m : Real) : Real;
begin
  dajB:=y-m*x;
end;

{ Funkcja sprawdza czy punkt naley do prostej }
{ Parametry wejciowe: m - wspczynnik kierunkowy, b - wyraz wolny prostej }
{ p - punkt }
function czy_do_prostej(m,b : Real; p : TPunkt) : Boolean;
var
  wynik : Boolean;
  y : Real;
begin
  wynik:=false;
  { 1. podstawiam wsprzdn x punktu p do rwnania prostej i otrzymuj y }
  { 2. porwnuj y z prostej ze wsprzdn y punktu p }
  y:=m*p.x + b;

  if y=p.y then wynik:=true;
  czy_do_prostej:=wynik;
end;

{ Funkcja bada przynaleno c do przedziau <a,b> }
function czy_do_przedzialu (c, a, b : Real) : Boolean;
begin
  czy_do_przedzialu:=(c>=min(a,b)) and (c<=max(a,b));
end;

{ Funkcja bada czy punkt naley do odcinka }
function czy_do_odcinka( o : TOdcinek; p : TPunkt) : Boolean;
var
  m : Real;
  wynik : Boolean;
begin
  wynik:=false;
  if not czy_do_przedzialu(p.x, o.p.x, o.q.x) then wynik:=false
  else
    if not czy_do_przedzialu(p.y, o.p.y, o.q.y) then wynik:=false
    else
      begin

        if o.n.dx=0 then { jeli odcinek pionowy }
          begin
            wynik:=true;
          end
        else
          begin
            m:=dajKier(o.n);
            wynik:=(sprawdz_nachyl(o.n.dy,o.n.dx)=2)
                   or
                   { sprawdzenie czy teoretyczny punkt naley do odcinka }
                   (p.y=(m*p.x + dajB(o.p.x,o.p.y,m)));
          end;
      end;
  czy_do_odcinka:=wynik;
end;

var
  p1, p2,
  p3     : TPunkt;
  n1     : TNachyl;
  o1     : TOdcinek;
  m, b   : Real;
  typN_o : Byte; { typ nachylenia odcinka }
  jest   : Boolean;
begin
  jest:=false;
  clrscr;
  writeln('Program sprawdza czy punkt naley do okrelonego odcinka');

  {I przypadek odcinka z nachyleniem }
  punkt(1,1,p1);  { pocztek odcinka }
  punkt(3,4,p2);  { koniec odcinka   }

(*
  {II przypadek odcinka rwnolegego do osi OX }
  punkt(0,1,p1);  { pocztek odcinka }
  punkt(2,1,p2);  { koniec odcinka   }

  {III przypadek odcinka rwnolegego do osi OY }
  punkt(2,0,p1);  { pocztek odcinka }
  punkt(2,1,p2);  { koniec odcinka   }
*)

  write('Pocztek odcinka: ');
  writeln('(',p1.x:6:2,',',p1.y:6:2,')');

  write('Koniec odcinka: ');
  writeln('(',p2.x:6:2,',',p2.y:6:2,')');

  writeln;
  { sprawdzamy czy odcinek nie jest zerowej dugoci }
  write('Odlego pomidzy pocztkiem a kocem odcinka wynosi: ');
  writeln(odl_pkt(p1,p2):6:2);


  writeln;
  writeln('Punkty:');

  {-----------------------------}
  { Dla I }
(*
  punkt(2,2,p3);    { Dla I - nie naley do prostej i nie naley do odcinka}
  punkt(2,2.5,p3);  { Dla I - naley do prostej i do odcinka }
*)
  punkt(0,-0.5,p3); { Dla I - naley do prostej, ale nie do odcinka}
(*
  {-----------------------------}
  { Dla II }
  punkt(1,1,p3);  { Dla II -  naley do prostej i do odcinka }

  {-----------------------------}
  { Dla III }
  punkt(2,2,p3);  {Dla III - naley do prostej, ale nie do odcinka }
*)
  writeln('p1 = (',p3.x:6:2,',',p3.y:6:2,')');

  writeln;

  n1.dy:=p2.y-p1.y;
  n1.dx:=p2.x-p1.x;

  { wypeniamy rekord, aby w jednym miejscu mie informacj o odcinku }
  { i w dalszej czci programu z niego bdziemy pobiera dane }
  odcinek(p1,p2,n1,o1);

  { sprawdzamy nachylenie odcinka }
  typN_o:=sprawdz_nachyl(o1.n.dy, o1.n.dx);

  case typN_o of
    255   : writeln('Nie jest to odcinek - dugo zerowa');
      0,1 : begin
              m:=dajKier(o1.n);
              b:=dajB(o1.p.x,o1.p.y,m);

             { korzystamy teraz z funkcji, ktra bada czy punkt naley do prostej }
              if czy_do_prostej(m,b,p3) then
                jest:=true
              else
                writeln('Punkt p1 znajduje si poza prost, czyli te poza odcinkiem');

            end;
      2   : { sprawdzamy czy punkt ley na prostej y=b }
            if (o1.p.x=p3.x) then jest:=true
            else
              writeln('Punkt p1 znajduje si poza prost, czyli te poza odcinkiem');

  end;

  if jest then
    begin
      write('Punkt p1 naley do prostej ');
      if czy_do_odcinka(o1,p3) then writeln('i take do odcinka' )
      else
        writeln('ale nie naley do odcinka');
    end;

  repeat until keypressed;
end.
W komentarzach umieszczone sa rzne przypadki punktw i odcinkw, ktre
moesz wykorzysta do testowania programu.
